@@ -9,7 +9,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -174,10 +174,8 @@ int addparty(char *bot, char *nick, int chan, char flag, int sock,
     party = nrealloc(party, party_size * sizeof(party_t));
     debug1("botnet: party size doubled to %i.", party_size);
   }
-  strncpy(party[parties].nick, nick, HANDLEN);
-  party[parties].nick[HANDLEN] = 0;
-  strncpy(party[parties].bot, bot, HANDLEN);
-  party[parties].bot[HANDLEN] = 0;
+  strlcpy(party[parties].nick, nick, HANDLEN + 1);
+  strlcpy(party[parties].bot, bot, HANDLEN + 1);
   party[parties].chan = chan;
   party[parties].sock = sock;
   party[parties].status = 0;
@@ -359,7 +357,7 @@ void remparty(char *bot, int sock)
 
 /* Cancel every user that was on a certain bot
  */
-void rempartybot(char *bot)
+static void rempartybot(char *bot)
 {
   int i;
 
@@ -1141,33 +1139,15 @@ static void failed_tandem_relay(int idx)
     lostdcc(idx);
     return;
   }
-  if (dcc[idx].port >= dcc[idx].u.relay->port + 3) {
-    struct chat_info *ci = dcc[uidx].u.relay->chat;
-
-    dprintf(uidx, "%s %s.\n", BOT_CANTLINKTO, dcc[idx].nick);
-    dcc[uidx].status = dcc[uidx].u.relay->old_status;
-    nfree(dcc[uidx].u.relay);
-    dcc[uidx].u.chat = ci;
-    dcc[uidx].type = &DCC_CHAT;
-    killsock(dcc[idx].sock);
-    lostdcc(idx);
-    return;
-  }
+  dprintf(uidx, "%s %s.\n", BOT_CANTLINKTO, dcc[idx].nick);
+  dcc[uidx].status = dcc[uidx].u.relay->old_status;
+  struct chat_info *ci = dcc[uidx].u.relay->chat;
+  nfree(dcc[uidx].u.relay);
+  dcc[uidx].u.chat = ci;
+  dcc[uidx].type = &DCC_CHAT;
   killsock(dcc[idx].sock);
-  if (!dcc[idx].sockname.addrlen)
-    (void) setsockname(&dcc[idx].sockname, dcc[idx].host, dcc[idx].port, 0);
-  dcc[idx].sock = getsock(dcc[idx].sockname.family, SOCK_STRONGCONN);
-  dcc[uidx].u.relay->sock = dcc[idx].sock;
-  dcc[idx].port++;
-  dcc[idx].timeval = now;
-  if (dcc[idx].sock < 0 ||
-      open_telnet_raw(dcc[idx].sock, &dcc[idx].sockname) < 0)
-    failed_tandem_relay(idx);
-#ifdef TLS
-  else if (dcc[idx].ssl && ssl_handshake(dcc[idx].sock, TLS_CONNECT,
-           tls_vfybots, LOG_BOTS, dcc[idx].host, NULL))
-    failed_tandem_relay(idx);
-#endif
+  lostdcc(idx);
+  return;
 }
 
 
