@@ -6,7 +6,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -60,6 +60,7 @@
 #define UHOSTMAX    291 + NICKMAX /* 32 (ident) + 3 (\0, !, @) + NICKMAX */
 #define DIRMAX      512           /* paranoia                            */
 #define LOGLINEMAX  9000          /* for misc.c/putlog() <cybah>         */
+#define READMAX     16384         /* for read() and SSL_read()           */
 
 /* Invalid characters */
 #define BADHANDCHARS "-,+*=:!.@#;$%&"
@@ -88,13 +89,6 @@
 #define PASSWORDMAX 30 /* highest value compatible to older eggdrop */
 #define PASSWORDLEN PASSWORDMAX + 1
 
-
-/* We have to generate compiler errors in a weird way since not all compilers
- * support the #error preprocessor directive. */
-#ifndef STDC_HEADERS
-#  include "Error: Your system must have standard ANSI C headers."
-#endif
-
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
 #endif
@@ -184,22 +178,18 @@
 #  endif
 #endif
 
-/* Almost every module needs some sort of time thingy, so... */
-#ifdef TIME_WITH_SYS_TIME
-#  include <sys/time.h>
-#  include <time.h>
-#else
-#  ifdef HAVE_SYS_TIME_H
-#    include <sys/time.h>
-#  else
-#    include <time.h>
-#  endif
+#ifndef PATH_MAX
+#  define PATH_MAX 4096
 #endif
 
+/* Almost every module needs some sort of time thingy, so... */
+#include <sys/time.h> /* gettimeofday() POSIX 2001 */
+#include <time.h> /* POSIX 2001 */
+#include <sys/resource.h> /* getrusage() setrlimit() after time.h because of BSD */
 
 /* Yikes...who would have thought finding a usable random() would be so much
  * trouble?
- * Note: random(), rand(), and lrand48() are *not* thread safe.
+ * Note: random() is *not* thread safe.
  *
  * QNX doesn't include random() and srandom() in libc.so, only in libc.a
  * So we can only use these functions in static builds on QNX.
@@ -209,49 +199,15 @@
 #  undef HAVE_SRANDOM
 #endif
 
-#ifdef HAVE_RANDOM
-  /* On systems with random(), RANDOM_MAX may or may not be defined.
-   *
-   * If RANDOM_MAX isn't defined, we use 0x7FFFFFFF (2^31-1), or 2147483647
-   * since this follows the 4.3BSD and POSIX.1-2001 standards. This of course
-   * assumes random() uses a 32 bit long int type per the standards.
-   */
-#  ifndef RANDOM_MAX
-#    define RANDOM_MAX 0x7FFFFFFF  /* random() -- 2^31-1 */
-#  endif
-#else                              /* !HAVE_RANDOM */
-   /* This shouldn't exist in this case, but just to be safe... */
-#  ifdef RANDOM_MAX
-#    undef RANDOM_MAX
-#  endif
-  /* If we don't have random() it's safe to assume we also don't have
-   * srandom(), and we need both.
-   */
-#  ifdef HAVE_RAND
-#    define random() rand()
-#    define srandom(x) srand(x)
-    /* Depending on the system int size, RAND_MAX can be either 0x7FFFFFFF
-     * (2^31-1), or 2147483647 for a 32 bit int, or 0x7FFF (2^15-1), or
-     * 32767 for a 16 bit int. The standards only state that RAND_MAX must
-     * be _at least_ 32767 but some systems with 16 bit int define it as
-     * 32767. See: SVr4, 4.3BSD, C89, C99, POSIX.1-2001.
-     */
-#    define RANDOM_MAX RAND_MAX    /* rand() -- 2^31-1 or 2^15-1 */
-#  else                            /* !HAVE_RAND */
-#    ifdef  HAVE_LRAND48
-#      define random() lrand48()
-#      define srandom(x) srand48(x)
-      /* For lrand48() we define RANDOM_MAX as 0x7FFFFFFF (2^31-1), or
-       * 2147483647 since this is what the SVr4 and POSIX.1-2001 standards
-       * call for. Note: SVID 3 declares these functions as obsolete and
-       * states rand() should be used instead.
-       */
-#      define RANDOM_MAX 0x7FFFFFFF /* lrand48() -- 2^31-1 */
-#    else                          /* !HAVE_LRAND48 */
-#      include "Error: Must define one of HAVE_RANDOM, HAVE_RAND, or HAVE_LRAND48"
-#    endif                         /* HAVE_LRAND48 */
-#  endif                           /* HAVE_RAND */
-#endif                             /* HAVE_RANDOM */
+/* On systems with random(), RANDOM_MAX may or may not be defined.
+ *
+ * If RANDOM_MAX isn't defined, we use 0x7FFFFFFF (2^31-1), or 2147483647
+ * since this follows the 4.3BSD and POSIX.1-2001 standards. This of course
+ * assumes random() uses a 32 bit long int type per the standards.
+ */
+#ifndef RANDOM_MAX
+#  define RANDOM_MAX 0x7FFFFFFF  /* random() -- 2^31-1 */
+#endif
 
 
 /* Use high-order bits for getting the random integer. With a modern
@@ -274,14 +230,6 @@
 #define nrealloc(x,y) n_realloc((x),(y),__FILE__,__LINE__)
 #define nfree(x)      n_free((x),__FILE__,__LINE__)
 
-#ifdef DEBUG_CONTEXT
-#  define Context           eggContext(__FILE__, __LINE__, NULL)
-#  define ContextNote(note) eggContextNote(__FILE__, __LINE__, NULL, note)
-#else
-#  define Context           do {} while (0)
-#  define ContextNote(note) do {} while (0)
-#endif
-
 #ifdef DEBUG_ASSERT
 #  define Assert(expr) do {                                             \
           if (!(expr))                                                  \
@@ -793,12 +741,13 @@ enum {
 
 /* linked list instead of array because of multi threading */
 struct dns_thread_node {
+  pthread_t thread_id;
   pthread_mutex_t mutex;
   int fildes[2];
   int type;
   sockname_t addr;
   char host[256];
-  int ok;
+  char strerror[3 * 64]; /* msg + gai_strerror() + strerror() */
   struct dns_thread_node *next;
 };
 
