@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,17 +48,6 @@
 #  endif
 #endif
 
-#ifdef TIME_WITH_SYS_TIME
-#  include <sys/time.h>
-#  include <time.h>
-#else
-#  ifdef HAVE_SYS_TIME_H
-#    include <sys/time.h>
-#  else
-#    include <time.h>
-#  endif
-#endif
-
 #include "filedb3.h"
 #include "filesys.h"
 #include "src/tandem.h"
@@ -217,11 +206,14 @@ static void dcc_files_pass(int idx, char *buf, int x)
  */
 static int got_files_cmd(int idx, char *msg)
 {
+  const char *filt;
   char *code;
 
-  strcpy(msg, check_tcl_filt(idx, msg));
-  if (!msg[0])
+  filt = check_tcl_filt(idx, msg);
+  if (!filt[0])
     return 1;
+  if (filt != msg)
+    strcpy(msg, filt);
   if (msg[0] == '.')
     msg++;
   code = newsplit(&msg);
@@ -230,13 +222,17 @@ static int got_files_cmd(int idx, char *msg)
 
 static void dcc_files(int idx, char *buf, int i)
 {
+  const char*filt;
+
   if (buf[0] && detect_dcc_flood(&dcc[idx].timeval, dcc[idx].u.file->chat,
       idx))
     return;
   dcc[idx].timeval = now;
-  strcpy(buf, check_tcl_filt(idx, buf));
-  if (!buf[0])
+  filt = check_tcl_filt(idx, buf);
+  if (!filt[0])
     return;
+  if (filt != buf)
+    strcpy(buf, filt);
   touch_laston(dcc[idx].user, "filearea", now);
   if (buf[0] == ',') {
     for (i = 0; i < dcc_total; i++) {
@@ -796,6 +792,7 @@ static void filesys_dcc_send_hostresolved(int i)
     /* Put uploads in a temp file first */
     dcc[i].u.xfer->f = tmpfile();
     if (dcc[i].u.xfer->f == NULL) {
+      debug1("filesys: filesys_dcc_send_hostresolved(): tmpfile(): error: %s", strerror(errno));
       dprintf(DP_HELP,
               "NOTICE %s :Can't create file `%s' (temp dir error)\n",
               dcc[i].nick, dcc[i].u.xfer->origname);
