@@ -3,7 +3,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,7 +39,11 @@
 #define HOOK_LOADED              13
 #define HOOK_BACKUP              14
 #define HOOK_DIE                 15
-#define REAL_HOOKS               16
+#define HOOK_PRE_SELECT          16
+#define HOOK_POST_SELECT         17
+
+#define REAL_HOOKS               18
+
 #define HOOK_SHAREOUT           105
 #define HOOK_SHAREIN            106
 #define HOOK_ENCRYPT_PASS       107
@@ -72,7 +76,7 @@
 /* Server */
 #define SERVER_BOTNAME            4
 #define SERVER_BOTUSERHOST        5
-#define SERVER_NICKLEN           38
+#define SERVER_NICKLEN           37
 /* IRC */
 #define IRC_RECHECK_CHANNEL       15
 #define IRC_RECHECK_CHANNEL_MODES 17
