@@ -11,7 +11,7 @@
  * 1.2a    1997-08-24      Minor fixes. [BB]
  */
 /*
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -73,17 +73,6 @@
 
 #include "src/mod/module.h"
 
-#ifdef TIME_WITH_SYS_TIME
-#  include <sys/time.h>
-#  include <time.h>
-#else
-#  ifdef HAVE_SYS_TIME_H
-#    include <sys/time.h>
-#  else
-#    include <time.h>
-#  endif
-#endif
-
 #include "src/users.h"
 #include "src/chan.h"
 #include "channels.mod/channels.h"
@@ -153,7 +142,7 @@ static int dcc_seen(struct userrec *u, int idx, char *par)
 static void do_seen(int idx, char *prefix, char *nick, char *hand,
                     char *channel, char *text)
 {
-  char stuff[512], word1[512], word2[512], whotarget[128], object[128],
+  char word1[512], word2[512], whotarget[128], object[128],
        whoredirect[512], *oix, *lastonplace = 0;
   struct userrec *urec;
   struct chanset_t *chan;
@@ -199,8 +188,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
         m = ismember(chan, object);
         if (m) {
           onchan = 1;
-          snprintf(stuff, sizeof stuff, "%s!%s", object, m->userhost);
-          urec = get_user_by_host(stuff);
+          urec = get_user_from_member(m);
           if (!urec || !strcasecmp(object, urec->handle))
             break;
           strcat(whoredirect, object);
@@ -254,7 +242,7 @@ static void do_seen(int idx, char *prefix, char *nick, char *hand,
       dprintf(idx, "%sYour what, %s?\n", prefix, nick);
       return;
     }
-    /* Do I even KNOW the requestor? */
+    /* Do I even KNOW the requester? */
     if (hand[0] == '*' || !hand[0]) {
       dprintf(idx,
               "%sI don't know you, %s, so I don't know about your %s.\n",
@@ -347,7 +335,7 @@ targetcont:
     if (m) {
       onchan = 1;
       snprintf(word1, sizeof word1, "%s!%s", whotarget, m->userhost);
-      urec = get_user_by_host(word1);
+      urec = get_user_from_member(m);
       if (!urec || !strcasecmp(whotarget, urec->handle))
         break;
       strcat(whoredirect, whotarget);
@@ -364,8 +352,7 @@ targetcont:
     while (chan) {
       m = chan->channel.member;
       while (m && m->nick[0]) {
-        snprintf(word2, sizeof word2, "%s!%s", m->nick, m->userhost);
-        urec = get_user_by_host(word2);
+        urec = get_user_from_member(m);
         if (urec && !strcasecmp(urec->handle, whotarget)) {
           strcat(whoredirect, whotarget);
           strcat(whoredirect, " is ");
