@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -509,3 +509,15 @@ void isupport_report(int idx, const char *prefix, int details)
   }
 }
 
+static const char *isupport_get_prefixchars(void)
+{
+  const char *str = isupport_get("PREFIX", strlen("PREFIX"));
+
+  if (str) {
+    str = strchr(str, ')');
+    if (str && str[1]) {
+      return str + 1;
+    }
+  }
+  return "+%@&~";
+}
