@@ -1,5 +1,6 @@
 /*
- * This module reports uptime information about your bot to http://uptime.eggheads.org. The
+ * This module reports uptime statistics to the uptime contest web
+ * site at https://www.eggheads.org/uptime/. The
  * purpose for this is to see how your bot rates against many others (including EnergyMechs
  * and Eggdrops) -- It is a fun little project, jointly run by Eggheads.org and EnergyMech.net.
  *
@@ -11,7 +12,7 @@
  */
 /*
  * Copyright (C) 2001 proton
- * Copyright (C) 2001 - 2023 Eggheads Development Team
+ * Copyright (C) 2001 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -110,7 +111,7 @@ static void uptime_report(int idx, int details)
   }
 }
 
-unsigned long get_ip()
+static unsigned long get_ip()
 {
   struct hostent *hp;
   IP ip;
@@ -130,7 +131,7 @@ unsigned long get_ip()
   return ip;
 }
 
-int init_uptime(void)
+static int init_uptime(void)
 {
   struct sockaddr_in sai;
   char x[64], *z = x;
@@ -169,7 +170,7 @@ int init_uptime(void)
 }
 
 
-int send_uptime(void)
+static int send_uptime(void)
 {
   struct sockaddr_in sai;
   struct stat st;
@@ -186,7 +187,7 @@ int send_uptime(void)
 
   uptimecount++;
   upPack.packets_sent = htonl(uptimecount); /* Tell the server how many
-					       uptime packets we've sent. */
+                                             * uptime packets we've sent. */
   upPack.now2 = htonl(time(NULL));
   upPack.ontime = 0;
 
