@@ -8,7 +8,7 @@
  *
  * Changes after Feb 23, 1999 Copyright Eggheads Development Team
  *
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 
 #include <fcntl.h>
 #include "main.h"
+#include "modules.h"
 #include <limits.h>
 #include <string.h>
 #include <netdb.h>
@@ -42,7 +43,6 @@
 #  include <unistd.h>
 #endif
 #include <setjmp.h>
-#include "mod/server.mod/server.h"
 
 #ifdef TLS
 #  include <openssl/err.h>
@@ -130,7 +130,7 @@ char *iptostr(struct sockaddr *sa)
  */
 int setsockname(sockname_t *addr, char *src, int port, int allowres)
 {
-  char *endptr, *src2 = src;;
+  char *endptr, *src2 = src;
   long val;
   IP ip;
   volatile int af = AF_UNSPEC;
@@ -483,6 +483,7 @@ static int proxy_connect(int sock, sockname_t *addr)
   sockname_t name;
   char host[121], s[256];
   int i, port, proxy;
+  struct threaddata *td = threaddata();
 
   if (!firewall[0])
     return -2;
@@ -505,7 +506,7 @@ static int proxy_connect(int sock, sockname_t *addr)
   if (connect(sock, &name.addr.sa, name.addrlen) < 0 && errno != EINPROGRESS)
     return -1;
   if (proxy == PROXY_SOCKS) {
-    for (i = 0; i < threaddata()->MAXSOCKS; i++)
+    for (i = 0; i < td->MAXSOCKS; i++)
       if (!(socklist[i].flags & SOCK_UNUSED) && socklist[i].sock == sock)
         socklist[i].flags |= SOCK_PROXYWAIT;    /* drummer */
     memcpy(host, &addr->addr.s4.sin_addr.s_addr, 4);
@@ -547,6 +548,7 @@ int open_telnet_raw(int sock, sockname_t *addr)
   fd_set sockset;
   struct timeval tv;
   int i, j, rc, res;
+  struct threaddata *td = threaddata();
 
   for (i = 0; i < dcc_total; i++)
     if (dcc[i].sock == sock) { /* Got idx from sock ? */
@@ -563,7 +565,7 @@ int open_telnet_raw(int sock, sockname_t *addr)
   if (bind(sock, &name.addr.sa, name.addrlen) < 0) {
     return -1;
   }
-  for (j = 0; j < threaddata()->MAXSOCKS; j++) {
+  for (j = 0; j < td->MAXSOCKS; j++) {
     if (!(socklist[j].flags & SOCK_UNUSED) && (socklist[j].sock == sock))
       socklist[j].flags = (socklist[j].flags & ~SOCK_VIRTUAL) | SOCK_CONNECT;
   }
@@ -892,12 +894,13 @@ int sockread(char *s, int *len, sock_list *slist, int slistmax, int tclonly)
   struct timeval t;
   fd_set fdr, fdw, fde;
   int i, x, maxfd_r, maxfd_w, maxfd_e;
-  int grab = 511, tclsock = -1, events = 0;
+  int grab = READMAX, tclsock = -1, events = 0;
   struct threaddata *td = threaddata();
   int maxfd;
 #ifdef EGG_TDNS
   int fd;
   struct dns_thread_node *dtn, *dtn_prev;
+  void *res;
 #endif
 
   maxfd_r = preparefdset(&fdr, slist, slistmax, tclonly, TCL_READABLE);
@@ -922,11 +925,13 @@ int sockread(char *s, int *len, sock_list *slist, int slistmax, int tclonly)
   t.tv_sec = td->blocktime.tv_sec;
   t.tv_usec = td->blocktime.tv_usec;
 
+  call_hook(HOOK_PRE_SELECT);
   x = select((SELECT_TYPE_ARG1) maxfd + 1,
              SELECT_TYPE_ARG234 (maxfd_r >= 0 ? &fdr : NULL),
              SELECT_TYPE_ARG234 (maxfd_w >= 0 ? &fdw : NULL),
              SELECT_TYPE_ARG234 (maxfd_e >= 0 ? &fde : NULL),
              SELECT_TYPE_ARG5 &t);
+  call_hook(HOOK_POST_SELECT);
   if (x == -1)
     return -2;                  /* socket error */
   if (x == 0)
@@ -936,8 +941,7 @@ int sockread(char *s, int *len, sock_list *slist, int slistmax, int tclonly)
     if (!tclonly && ((!(slist[i].flags & (SOCK_UNUSED | SOCK_TCL))) &&
         ((FD_ISSET(slist[i].sock, &fdr)) ||
 #ifdef TLS
-        (slist[i].ssl && (SSL_pending(slist[i].ssl) ||
-         !SSL_is_init_finished(slist[i].ssl))) ||
+        (slist[i].ssl && !SSL_is_init_finished(slist[i].ssl)) ||
 #endif
         ((slist[i].sock == STDOUT) && (!backgrd) &&
          (FD_ISSET(STDIN, &fdr)))))) {
@@ -972,7 +976,12 @@ int sockread(char *s, int *len, sock_list *slist, int slistmax, int tclonly)
       {
         if (slist[i].ssl) {
           x = SSL_read(slist[i].ssl, s, grab);
-          if (x < 0) {
+          if (!x && (SSL_get_shutdown(slist[i].ssl) == SSL_RECEIVED_SHUTDOWN)) {
+            *len = slist[i].sock;
+            slist[i].flags &= ~SOCK_CONNECT;
+            debug1("net: SSL_read(): received shutdown sock %i", slist[i].sock);
+            return -1;
+          } else if (x < 0) {
             int err = SSL_get_error(slist[i].ssl, x);
             if (err == SSL_ERROR_WANT_READ || err == SSL_ERROR_WANT_WRITE)
               errno = EAGAIN;
@@ -1052,23 +1061,24 @@ int sockread(char *s, int *len, sock_list *slist, int slistmax, int tclonly)
 #ifdef EGG_TDNS
   dtn_prev = dns_thread_head;
   for (dtn = dtn_prev->next; dtn; dtn = dtn->next) {
+    pthread_mutex_lock(&dtn->mutex);
+    if (*dtn->strerror)
+      debug2("%s: hostname %s", dtn->strerror, dtn->host);
     fd = dtn->fildes[0];
     if (FD_ISSET(fd, &fdr)) {
-      if (dtn->type == DTN_TYPE_HOSTBYIP) {
-        pthread_mutex_lock(&dtn->mutex);
-        call_hostbyip(&dtn->addr, dtn->host, dtn->ok);
-        pthread_mutex_unlock(&dtn->mutex);
-      }
-      else {
-        pthread_mutex_lock(&dtn->mutex);
-        call_ipbyhost(dtn->host, &dtn->addr, dtn->ok);
-        pthread_mutex_unlock(&dtn->mutex);
-      }
-      close(dtn->fildes[0]);
+      if (dtn->type == DTN_TYPE_HOSTBYIP)
+        call_hostbyip(&dtn->addr, dtn->host, !*dtn->strerror);
+      else
+        call_ipbyhost(dtn->host, &dtn->addr, !*dtn->strerror);
+      pthread_mutex_unlock(&dtn->mutex);
+      close(fd);
+      if (pthread_join(dtn->thread_id, &res))
+        putlog(LOG_MISC, "*", "sockread(): pthread_join(): error = %s", strerror(errno));
       dtn_prev->next = dtn->next;
       nfree(dtn);
       dtn = dtn_prev;
-    }
+    } else
+      pthread_mutex_unlock(&dtn->mutex);
     dtn_prev = dtn;
   }
 #endif
@@ -1097,14 +1107,14 @@ int sockread(char *s, int *len, sock_list *slist, int slistmax, int tclonly)
  * dcc functions. Simply ignore it.
  * Returns -5 if tcl sockets are busy but not eggdrop sockets.
  */
-
 int sockgets(char *s, int *len)
 {
-  char xx[RECVLINEMAX], *p, *px, *p2;
+  char xx[READMAX + 2], *p, *px, *p2;
   int ret, i, data = 0;
   size_t len2;
+  struct threaddata *td = threaddata();
 
-  for (i = 0; i < threaddata()->MAXSOCKS; i++) {
+  for (i = 0; i < td->MAXSOCKS; i++) {
     /* Check for stored-up data waiting to be processed */
     if (!(socklist[i].flags & (SOCK_UNUSED | SOCK_TCL | SOCK_BUFFER)) &&
         (socklist[i].handler.sock.inbuf != NULL)) {
@@ -1126,7 +1136,7 @@ int sockgets(char *s, int *len)
             p++;
           *p2 = 0;
 
-          strlcpy(s, socklist[i].handler.sock.inbuf, RECVLINEMAX-1);
+          strlcpy(s, socklist[i].handler.sock.inbuf, READMAX + 1);
           if (*p) {
             len2 = strlen(p) + 1;
             px = nmalloc(len2);
@@ -1142,15 +1152,15 @@ int sockgets(char *s, int *len)
         }
       } else {
         /* Handling buffered binary data (must have been SOCK_BUFFER before). */
-        if (socklist[i].handler.sock.inbuflen <= RECVLINEMAX-2) {
+        if (socklist[i].handler.sock.inbuflen <= READMAX) {
           *len = socklist[i].handler.sock.inbuflen;
           memcpy(s, socklist[i].handler.sock.inbuf, socklist[i].handler.sock.inbuflen);
           nfree(socklist[i].handler.sock.inbuf);
           socklist[i].handler.sock.inbuf = NULL;
           socklist[i].handler.sock.inbuflen = 0;
         } else {
-          /* Split up into chunks of RECVLINEMAX-2 bytes. */
-          *len = RECVLINEMAX-2;
+          /* Split up into chunks of READMAX bytes. */
+          *len = READMAX;
           memcpy(s, socklist[i].handler.sock.inbuf, *len);
           memcpy(socklist[i].handler.sock.inbuf, socklist[i].handler.sock.inbuf + *len, *len);
           socklist[i].handler.sock.inbuflen -= *len;
@@ -1168,7 +1178,7 @@ int sockgets(char *s, int *len)
   }
   /* No pent-up data of any worth -- down to business */
   *len = 0;
-  ret = sockread(xx, len, socklist, threaddata()->MAXSOCKS, 0);
+  ret = sockread(xx, len, socklist, td->MAXSOCKS, 0);
   if (ret < 0) {
     s[0] = 0;
     return ret;
@@ -1199,7 +1209,7 @@ int sockgets(char *s, int *len)
     return socklist[ret].sock;
   }
   if (socklist[ret].flags & SOCK_BUFFER) {
-    socklist[ret].handler.sock.inbuf = (char *) nrealloc(socklist[ret].handler.sock.inbuf,
+    socklist[ret].handler.sock.inbuf = nrealloc(socklist[ret].handler.sock.inbuf,
                                             socklist[ret].handler.sock.inbuflen + *len + 1);
     memcpy(socklist[ret].handler.sock.inbuf + socklist[ret].handler.sock.inbuflen, xx, *len);
     socklist[ret].handler.sock.inbuflen += *len;
@@ -1211,21 +1221,22 @@ int sockgets(char *s, int *len)
   /* Might be necessary to prepend stored-up data! */
   if (socklist[ret].handler.sock.inbuf != NULL) {
     p = socklist[ret].handler.sock.inbuf;
-    socklist[ret].handler.sock.inbuf = nmalloc(strlen(p) + strlen(xx) + 1);
-    strcpy(socklist[ret].handler.sock.inbuf, p);
-    strcat(socklist[ret].handler.sock.inbuf, xx);
+    len2 = strlen(p);
+    socklist[ret].handler.sock.inbuf = nmalloc(len2 + strlen(xx) + 1);
+    memcpy(socklist[ret].handler.sock.inbuf, p, len2);
+    strcpy(socklist[ret].handler.sock.inbuf + len2, xx);
     nfree(p);
-    if (strlen(socklist[ret].handler.sock.inbuf) < RECVLINEMAX) {
+    if (strlen(socklist[ret].handler.sock.inbuf) < READMAX + 2) {
       strcpy(xx, socklist[ret].handler.sock.inbuf);
       nfree(socklist[ret].handler.sock.inbuf);
       socklist[ret].handler.sock.inbuf = NULL;
       socklist[ret].handler.sock.inbuflen = 0;
     } else {
       p = socklist[ret].handler.sock.inbuf;
-      socklist[ret].handler.sock.inbuflen = strlen(p) - RECVLINEMAX-2;
+      socklist[ret].handler.sock.inbuflen = strlen(p) - READMAX;
       socklist[ret].handler.sock.inbuf = nmalloc(socklist[ret].handler.sock.inbuflen + 1);
-      strcpy(socklist[ret].handler.sock.inbuf, p + RECVLINEMAX-2);
-      *(p + RECVLINEMAX-2) = 0;
+      strcpy(socklist[ret].handler.sock.inbuf, p + READMAX);
+      *(p + READMAX) = 0;
       strcpy(xx, p);
       nfree(p);
       /* (leave the rest to be post-pended later) */
@@ -1246,7 +1257,7 @@ int sockgets(char *s, int *len)
 /* if (!s[0]) strcpy(s," ");  */
   if (!data) { 
     s[0] = 0;
-    if (strlen(xx) >= RECVLINEMAX-2) {
+    if (strlen(xx) >= READMAX) {
       /* String is too long, so just insert fake \n */
       strcpy(s, xx);
       xx[0] = 0;
@@ -1264,10 +1275,11 @@ int sockgets(char *s, int *len)
   /* Prepend old data back */
   if (socklist[ret].handler.sock.inbuf != NULL) {
     p = socklist[ret].handler.sock.inbuf;
-    socklist[ret].handler.sock.inbuflen = strlen(p) + strlen(xx);
+    len2 = strlen(xx);
+    socklist[ret].handler.sock.inbuflen = len2 + strlen(p);
     socklist[ret].handler.sock.inbuf = nmalloc(socklist[ret].handler.sock.inbuflen + 1);
-    strcpy(socklist[ret].handler.sock.inbuf, xx);
-    strcat(socklist[ret].handler.sock.inbuf, p);
+    memcpy(socklist[ret].handler.sock.inbuf, xx, len2);
+    strcpy(socklist[ret].handler.sock.inbuf + len2, p);
     nfree(p);
   } else {
     socklist[ret].handler.sock.inbuflen = strlen(xx);
@@ -1282,14 +1294,13 @@ int sockgets(char *s, int *len)
 }
 
 /* Dump something to a socket
- *
- * NOTE: Do NOT put Contexts in here if you want DEBUG to be meaningful!!
  */
 void tputs(int z, char *s, unsigned int len)
 {
   int i, x, idx;
   char *p;
   static int inhere = 0;
+  struct threaddata *td = threaddata();
 
   if (z < 0) /* um... HELLO?! sanity check please! */
     return;
@@ -1299,7 +1310,7 @@ void tputs(int z, char *s, unsigned int len)
     return;
   }
 
-  for (i = 0; i < threaddata()->MAXSOCKS; i++) {
+  for (i = 0; i < td->MAXSOCKS; i++) {
     if (!(socklist[i].flags & SOCK_UNUSED) && (socklist[i].sock == z)) {
       for (idx = 0; idx < dcc_total; idx++) {
         if ((dcc[idx].sock == z) && dcc[idx].type && dcc[idx].type->name) {
@@ -1325,7 +1336,7 @@ void tputs(int z, char *s, unsigned int len)
 
       if (socklist[i].handler.sock.outbuf != NULL) {
         /* Already queueing: just add it */
-        p = (char *) nrealloc(socklist[i].handler.sock.outbuf, socklist[i].handler.sock.outbuflen + len);
+        p = nrealloc(socklist[i].handler.sock.outbuf, socklist[i].handler.sock.outbuflen + len);
         memcpy(p + socklist[i].handler.sock.outbuflen, s, len);
         socklist[i].handler.sock.outbuf = p;
         socklist[i].handler.sock.outbuflen += len;
@@ -1382,13 +1393,14 @@ void dequeue_sockets()
   fd_set wfds;
   struct timeval tv;
   int maxfd = -1;
+  struct threaddata *td = threaddata();
 
 /* ^-- start poptix test code, this should avoid writes to sockets not ready to be written to. */
 
   FD_ZERO(&wfds);
   tv.tv_sec = 0;
   tv.tv_usec = 0;               /* we only want to see if it's ready for writing, no need to actually wait.. */
-  for (i = 0; i < threaddata()->MAXSOCKS; i++)
+  for (i = 0; i < td->MAXSOCKS; i++)
     if (!(socklist[i].flags & (SOCK_UNUSED | SOCK_TCL)) &&
         (socklist[i].handler.sock.outbuf != NULL)) {
       if (socklist[i].sock > maxfd)
@@ -1407,9 +1419,24 @@ void dequeue_sockets()
   if (x <= 0)
     return;
 
-  for (i = 0; i < threaddata()->MAXSOCKS; i++) {
+  for (i = 0; i < td->MAXSOCKS; i++) {
     if (!(socklist[i].flags & (SOCK_UNUSED | SOCK_TCL)) &&
         (socklist[i].handler.sock.outbuf != NULL) && (FD_ISSET(socklist[i].sock, &wfds))) {
+#ifdef CYGWIN_HACKS
+      int res;
+      socklen_t res_len;
+      if (socklist[i].flags == SOCK_CONNECT) {
+        res_len = sizeof(res);
+        getsockopt(socklist[i].sock, SOL_SOCKET, SO_ERROR, &res, &res_len);
+        if (res == ECONNREFUSED) { /* Connection refused */
+          int idx = findanyidx(socklist[i].sock);
+          putlog(LOG_MISC, "*", "Connection refused: %s:%i", dcc[idx].host,
+                 dcc[idx].port);
+          socklist[i].flags |= SOCK_EOFD;
+          return ;
+        }
+      }
+#endif
       /* Trick tputs into doing the work */
       errno = 0;
 #ifdef TLS
@@ -1482,9 +1509,10 @@ void tell_netdebug(int idx)
 {
   int i;
   char s[80];
+  struct threaddata *td = threaddata();
 
   dprintf(idx, "Open sockets:");
-  for (i = 0; i < threaddata()->MAXSOCKS; i++) {
+  for (i = 0; i < td->MAXSOCKS; i++) {
     if (!(socklist[i].flags & SOCK_UNUSED)) {
       sprintf(s, " %d", socklist[i].sock);
       if (socklist[i].flags & SOCK_BINARY)
@@ -1610,11 +1638,12 @@ int hostsanitycheck_dcc(char *nick, char *from, sockname_t *ip, char *dnsname,
 int sock_has_data(int type, int sock)
 {
   int ret = 0, i;
+  struct threaddata *td = threaddata();
 
-  for (i = 0; i < threaddata()->MAXSOCKS; i++)
+  for (i = 0; i < td->MAXSOCKS; i++)
     if (!(socklist[i].flags & SOCK_UNUSED) && socklist[i].sock == sock)
       break;
-  if (i < threaddata()->MAXSOCKS) {
+  if (i < td->MAXSOCKS) {
     switch (type) {
     case SOCK_DATA_OUTGOING:
       ret = (socklist[i].handler.sock.outbuf != NULL);
@@ -1642,9 +1671,10 @@ int flush_inbuf(int idx)
 {
   int i, len;
   char *inbuf;
+  struct threaddata *td = threaddata();
 
   Assert((idx >= 0) && (idx < dcc_total));
-  for (i = 0; i < threaddata()->MAXSOCKS; i++) {
+  for (i = 0; i < td->MAXSOCKS; i++) {
     if ((dcc[idx].sock == socklist[i].sock) &&
         !(socklist[i].flags & SOCK_UNUSED)) {
       len = socklist[i].handler.sock.inbuflen;
@@ -1687,8 +1717,9 @@ char *traced_myiphostname(ClientData cd, Tcl_Interp *irp, EGG_CONST char *name1,
 {
   const char *value;
 
-  if (flags & TCL_INTERP_DESTROYED)
+  if (Tcl_InterpDeleted(irp))
     return NULL;
+
   /* Recover trace in case of unset. */
   if (flags & TCL_TRACE_DESTROYED) {
     Tcl_TraceVar2(irp, name1, name2, TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS, traced_myiphostname, cd);
