@@ -6,7 +6,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -86,9 +86,7 @@ int remote_boots = 2;
 int allow_dk_cmds = 1;
 int must_be_owner = 1;
 int quiet_reject = 1;
-int copy_to_tmp = 1;
 int max_socks = 100;
-int quick_logs = 0;
 int par_telnet_flood = 1;
 int quiet_save = 0;
 int strtot = 0;
@@ -185,9 +183,10 @@ static char *tcl_eggint(ClientData cdata, Tcl_Interp *irp,
                         EGG_CONST char *name1,
                         EGG_CONST char *name2, int flags)
 {
-  char *s, s1[40];
+  char *s, s1[40], *endptr;
   long l;
   intinfo *ii = (intinfo *) cdata;
+  int p;
 
   if (flags & (TCL_TRACE_READS | TCL_TRACE_UNSETS)) {
     /* Special cases */
@@ -225,10 +224,9 @@ static char *tcl_eggint(ClientData cdata, Tcl_Interp *irp,
 
         default_uflags = fr.udef_global;
       } else if ((int *) ii->var == &userfile_perm) {
-        int p = oatoi(s);
-
-        if (p <= 0)
-          return "Invalid userfile permissions";
+        p = strtol(s, &endptr, 8);
+        if ((p < 01) || (p > 0777) || (*endptr))
+          return "Invalid userfile permissions, must be octal between 01 and 0777";
         userfile_perm = p;
       } else if ((ii->ro == 2) || ((ii->ro == 1) && protect_readonly))
         return "Read-only variable";
@@ -481,7 +479,6 @@ static tcl_ints def_tcl_ints[] = {
   {"max-socks",             &max_socks,            0},
   {"max-logs",              &max_logs,             0},
   {"max-logsize",           &max_logsize,          0},
-  {"quick-logs",            &quick_logs,           0},
   {"raw-log",               &raw_log,              1},
   {"protect-telnet",        &protect_telnet,       0},
   {"dcc-sanitycheck",       &dcc_sanitycheck,      0},
@@ -498,7 +495,6 @@ static tcl_ints def_tcl_ints[] = {
   {"force-expire",          &force_expire,         0},
   {"dupwait-timeout",       &dupwait_timeout,      0},
   {"userfile-perm",         &userfile_perm,        0},
-  {"copy-to-tmp",           &copy_to_tmp,          0},
   {"quiet-reject",          &quiet_reject,         0},
   {"cidr-support",          &cidr_support,         0},
   {"remove-pass",           &remove_pass,          0},
@@ -771,6 +767,7 @@ Tcl_Obj *egg_string_unicodesup_desurrogate(const char *oldstr, int len)
 {
   int stridx = 0, bufidx = 0;
   char *buf = nmalloc(len);
+  Tcl_Obj *o;
 
   while (stridx < len) {
     uint32_t low, high;
@@ -791,7 +788,10 @@ Tcl_Obj *egg_string_unicodesup_desurrogate(const char *oldstr, int len)
       }
     }
   }
-  return Tcl_NewStringObj(buf, bufidx);
+
+  o = Tcl_NewStringObj(buf, bufidx);
+  nfree(buf);
+  return o;
 }
 
 /* C function called for ::egg_tcl_tolower/toupper/totitle
@@ -905,17 +905,10 @@ void init_unicodesup(void)
 }
 #endif /* TCL_WORKAROUND_UNICODESUP */
 
-/* Not going through Tcl's crazy main() system (what on earth was he
- * smoking?!) so we gotta initialize the Tcl interpreter
- */
-void init_tcl(int argc, char **argv)
+void init_tcl0(int argc, char **argv)
 {
   Tcl_NotifierProcs notifierprocs;
-
-  const char *encoding;
-  int i, j;
-  char *langEnv, pver[1024] = "";
-
+ 
   egg_bzero(&notifierprocs, sizeof(notifierprocs));
   notifierprocs.initNotifierProc = tickle_InitNotifier;
   notifierprocs.createFileHandlerProc = tickle_CreateFileHandler;
@@ -927,8 +920,8 @@ void init_tcl(int argc, char **argv)
   notifierprocs.serviceModeHookProc = tickle_ServiceModeHook;
 
   Tcl_SetNotifier(&notifierprocs);
-
-/* This must be done *BEFORE* Tcl_SetSystemEncoding(),
+  
+  /* This must be done *BEFORE* Tcl_SetSystemEncoding(),
  * or Tcl_SetSystemEncoding() will cause a segfault.
  */
   /* This is used for 'info nameofexecutable'.
@@ -936,6 +929,19 @@ void init_tcl(int argc, char **argv)
    * the environment variable PATH for it to register anything.
    */
   Tcl_FindExecutable(argv[0]);
+#if TCL_MAJOR_VERSION >= 9
+  Tcl_InitSubsystems();
+#endif
+}
+
+/* Not going through Tcl's crazy main() system (what on earth was he
+ * smoking?!) so we gotta initialize the Tcl interpreter
+ */
+void init_tcl1(int argc, char **argv)
+{
+  const char *encoding;
+  int i, j;
+  char *langEnv, pver[1024] = "";
 
   /* Initialize the interpreter */
   interp = Tcl_CreateInterp();
@@ -1043,7 +1049,6 @@ resetPath:
   add_tcl_commands(tcluser_cmds);
   add_tcl_commands(tcldcc_cmds);
   add_tcl_commands(tclmisc_cmds);
-  add_tcl_objcommands(tclmisc_objcmds);
   add_tcl_commands(tcldns_cmds);
 #ifdef TLS
   add_tcl_commands(tcltls_cmds);
@@ -1247,17 +1252,17 @@ time_t get_expire_time(Tcl_Interp * irp, const char *s) {
   long expire_foo = strtol(s, &endptr, 10);
 
   if (*endptr) {
-    Tcl_AppendResult(irp, "bogus expire time", NULL);
+    Tcl_SetResult(irp, "bogus expire time", TCL_STATIC);
     return -1;
   }
   if (expire_foo < 0) {
-    Tcl_AppendResult(irp, "expire time must be 0 (perm) or greater than 0 days", NULL);
+    Tcl_SetResult(irp, "expire time must be 0 (perm) or greater than 0 days", TCL_STATIC);
     return -1;
   }
   if (expire_foo == 0)
     return 0;
   if (expire_foo > (60 * 24 * 2000)) {
-    Tcl_AppendResult(irp, "expire time must be equal to or less than 2000 days", NULL);
+    Tcl_SetResult(irp, "expire time must be equal to or less than 2000 days", TCL_STATIC);
     return -1;
   }
   return now + 60 * expire_foo;
