@@ -3,7 +3,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -79,7 +79,10 @@ tcl_bind_list_t *add_bind_table(const char *nme, int flg, IntFunc func);
 void del_bind_table(tcl_bind_list_t *tl_which);
 
 tcl_bind_list_t *find_bind_table(const char *nme);
-
+int bind_bind_entry(tcl_bind_list_t *tl, const char *flags,
+                           const char *cmd, const char *proc);
+int unbind_bind_entry(tcl_bind_list_t *tl, const char *flags,
+                             const char *cmd, const char *proc);
 int check_tcl_bind(tcl_bind_list_t *, const char *, struct flag_record *,
                    const char *, int);
 int check_tcl_dcc(const char *, int, const char *);
@@ -97,6 +100,7 @@ void check_tcl_nkch(const char *, const char *);
 void check_tcl_away(const char *, int, const char *);
 void check_tcl_chatactbcst(const char *, int, const char *, tcl_bind_list_t *);
 void check_tcl_event(const char *);
+void check_tcl_event_arg(const char *, const char *);
 int check_tcl_signal(const char *);
 void check_tcl_die(char *);
 void check_tcl_log(int, char *, char *);
