@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,20 +24,7 @@
 #include <errno.h>
 #include "main.h"
 #include "modules.h"
-#include "tandem.h"
 #include "md5/md5.h"
-
-#ifdef TIME_WITH_SYS_TIME
-#  include <sys/time.h>
-#  include <time.h>
-#else
-#  ifdef HAVE_SYS_TIME_H
-#    include <sys/time.h>
-#  else
-#    include <time.h>
-#  endif
-#endif
-
 #include <sys/stat.h>
 #include <sys/utsname.h>
 
@@ -247,7 +234,7 @@ static int tcl_binds STDVAR
   return TCL_OK;
 }
 
-int check_timer_syntax(Tcl_Interp *irp, int argc, char *argv[]) {
+int check_timer_syntax(Tcl_Interp *irp, int argc, char *argv[], tcl_timer_t *stack) {
   char *endptr;
   long val;
 
@@ -270,7 +257,7 @@ int check_timer_syntax(Tcl_Interp *irp, int argc, char *argv[]) {
         return 1;
       }
       /* Check for existing timers by same name */
-      if (find_timer(timer, argv[4])) {
+      if (find_timer(stack, argv[4])) {
         Tcl_AppendResult(irp, "timer already exists by that name", NULL);
         return 1;
       }
@@ -285,7 +272,7 @@ static int tcl_timer STDVAR
 
   BADARGS(3, 5, " minutes command ?count ?name??");
 
-  if (check_timer_syntax(irp, argc, argv)) {
+  if (check_timer_syntax(irp, argc, argv, timer)) {
     return TCL_ERROR;
   }
   x = add_timer(&timer, atoi(argv[1]), (argc >= 4 ? atoi(argv[3]) : 1),
@@ -304,7 +291,7 @@ static int tcl_utimer STDVAR
 
   BADARGS(3, 5, " seconds command ?count ?name??");
 
-  if (check_timer_syntax(irp, argc, argv)) {
+  if (check_timer_syntax(irp, argc, argv, utimer)) {
     return TCL_ERROR;
   }
   x = add_timer(&utimer, atoi(argv[1]), (argc == 4 ? atoi(argv[3]) : 1),
@@ -651,9 +638,13 @@ static int tcl_reloadhelp STDVAR
 
 static int tcl_callevent STDVAR
 {
-  BADARGS(2, 2, " event");
+  BADARGS(2, 3, " event ?arg?");
 
-  check_tcl_event(argv[1]);
+  if (argc == 2) {
+    check_tcl_event(argv[1]);
+  } else {
+    check_tcl_event_arg(argv[1], argv[2]);
+  }
   return TCL_OK;
 }
 
@@ -705,26 +696,29 @@ static int tcl_stripcodes STDVAR
   return TCL_OK;
 }
 
-static int tcl_md5(cd, irp, objc, objv)
-ClientData cd;
-Tcl_Interp *irp;
-int objc;
-Tcl_Obj *CONST objv[];
+static int tcl_md5 STDVAR
 {
-  MD5_CTX md5context;
-  char digest_string[33], *string;
+  char digest_string[33];
   unsigned char digest[16];
-  int i, len;
+  int i;
 
-  if (objc != 2) {
-    Tcl_WrongNumArgs(irp, 1, objv, "string");
-    return TCL_ERROR;
-  }
-  string = Tcl_GetStringFromObj(objv[1], &len);
+  BADARGS(2, 2, " string");
 
+#if (OPENSSL_VERSION_NUMBER >= 0x10100000L) && defined(HAVE_EVP_MD5)
+  EVP_MD_CTX *mdctx = EVP_MD_CTX_new();
+  const EVP_MD *md = EVP_md5();
+  unsigned int md_len;
+  EVP_DigestInit_ex(mdctx, md, NULL);
+  EVP_DigestUpdate(mdctx, argv[1], strlen(argv[1]));
+  EVP_DigestFinal_ex(mdctx, digest, &md_len);
+  EVP_MD_CTX_free(mdctx);
+#else
+  MD5_CTX md5context;
   MD5_Init(&md5context);
-  MD5_Update(&md5context, (unsigned char *) string, len);
+  MD5_Update(&md5context, (unsigned char *) argv[1], strlen(argv[1]));
   MD5_Final(digest, &md5context);
+#endif
+
   for (i = 0; i < 16; i++)
     sprintf(digest_string + (i * 2), "%.2x", digest[i]);
   Tcl_AppendResult(irp, digest_string, NULL);
@@ -764,11 +758,6 @@ static int tcl_matchstr STDVAR
   return TCL_OK;
 }
 
-tcl_cmds tclmisc_objcmds[] = {
-  {"md5", tcl_md5},
-  {NULL,     NULL}
-};
-
 static int tcl_status STDVAR
 {
   char s[15];
@@ -860,5 +849,6 @@ tcl_cmds tclmisc_cmds[] = {
   {"matchstr",         tcl_matchstr},
   {"status",             tcl_status},
   {"rfcequal",         tcl_rfcequal},
+  {"md5",                   tcl_md5},
   {NULL,                       NULL}
 };
