@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -1036,7 +1036,8 @@ static int xtra_pack(struct userrec *u, struct user_entry *e)
 
 static void xtra_display(int idx, struct user_entry *e)
 {
-  int code, lc, j;
+  int code;
+  Tcl_Size lc, j;
   EGG_CONST char **list;
   struct xtra_key *xk;
 
