@@ -12,7 +12,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,11 +30,10 @@
  */
 
 #include "main.h"
-#include "users.h"
-#include "chan.h"
 #include "modules.h"
 #include "tandem.h"
 
+#include <errno.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
@@ -144,7 +143,7 @@ void addignore(char *ign, char *from, char *mnote, time_t expire_time)
 }
 
 /* take host entry from ignore list and display it ignore-style */
-void display_ignore(int idx, int number, struct igrec *ignore)
+static void display_ignore(int idx, int number, struct igrec *ignore)
 {
   char dates[81], s[41];
 
@@ -524,8 +523,6 @@ void tell_user_ident(int idx, char *id)
   struct userrec *u;
 
   u = get_user_by_handle(userlist, id);
-  if (u == NULL)
-    u = get_user_by_host(id);
   if (u == NULL) {
     dprintf(idx, "%s.\n", USERF_NOMATCH);
     return;
@@ -698,9 +695,10 @@ int readuserfile(char *file, struct userrec **ret)
     global_invites = NULL;
   }
   lasthand[0] = 0;
-  f = fopen(file, "r");
-  if (f == NULL)
+  if (!(f = fopen(file, "r"))) {
+    debug2("users: fopen(%s): %s", file, strerror(errno));
     return 0;
+  }
   noshare = noxtra = 1;
   /* read opening comment */
   s = buf;
@@ -1031,7 +1029,7 @@ void autolink_cycle(char *start)
   }                             /* new run through the user list */
   while (u && !autc) {
     while (u && !autc) {
-      if (u->flags & USER_BOT && strcmp(u->handle, botnetnick)) {              /* ignore our own user record */
+      if (u->flags & USER_BOT && strcasecmp(u->handle, botnetnick)) {              /* ignore our own user record */
         bfl = bot_flags(u);
         if (bfl & (BOT_HUB | BOT_ALT)) {
           linked = 0;
