@@ -4,7 +4,7 @@
  */
 /*
  * Copyright (C) 1997 Robey Pointer
- * Copyright (C) 1999 - 2023 Eggheads Development Team
+ * Copyright (C) 1999 - 2024 Eggheads Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -180,13 +180,14 @@ extern struct igrec *global_ign;
  * Note: Flags are in eggdrop.h
  */
 
-struct userrec *adduser();
 struct userrec *get_user_by_handle(struct userrec *, char *);
 struct userrec *get_user_by_host(char *);
 struct userrec *get_user_by_account(char *);
 struct userrec *get_user_by_nick(char *);
+struct userrec *get_user_from_member(memberlist *);
+struct userrec *lookup_user_record(memberlist *, char *, char *);
 struct userrec *check_chanlist(const char *);
-struct userrec *check_chanlist_hand(const char *);
+memberlist *find_member_from_nick(char *);
 
 /* All the default userentry stuff, for code re-use
  */
